/* Copyright 2013 Sven Jacobs
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.svenjacobs.lugaene;

/*
 * #%L
 * LuGAEne
 * %%
 * Copyright (C) 2013 Sven Jacobs
 * %%
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * #L%
 */


import com.googlecode.objectify.Key;
import com.googlecode.objectify.annotation.Entity;
import com.googlecode.objectify.annotation.Id;
import com.googlecode.objectify.annotation.Parent;

/**
 * Represents a buffer of a {@link GaeFileEntity}
 * <p/>
 * A {@link GaeFileEntity} might own several buffers. Due tue GAE's entity size limitation of 1MB file buffers need to
 * be split into GaeFileBufferEntity instances.
 *
 * @author Sven Jacobs
 */
@Entity
class GaeFileBufferEntity {

    static final int BUFFER_SIZE = 1024;

    @Parent
    Key<GaeFileEntity> file;

    @Id
    Long index; // Index is not zero-based because it's an ID

    byte[] buffer = new byte[BUFFER_SIZE];

    @SuppressWarnings("UnusedDeclaration")
    private GaeFileBufferEntity() {
    }

    GaeFileBufferEntity(final GaeFileEntity file,
                        final long index) {
        this.file = Key.create(file);
        this.index = index;
    }
}
