/* Copyright 2013 Sven Jacobs
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.svenjacobs.lugaene;

/*
 * #%L
 * LuGAEne
 * %%
 * Copyright (C) 2013 Sven Jacobs
 * %%
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * #L%
 */


import java.util.HashMap;
import java.util.Map;

import static com.googlecode.objectify.ObjectifyService.ofy;
import static com.svenjacobs.lugaene.GaeFileBufferEntity.BUFFER_SIZE;

/**
 * Mixin with common functionality for {@link GaeIndexInput} and {@link GaeIndexOutput}
 *
 * @author Sven Jacobs
 */
class GaeIndexMixin {

    final GaeFileEntity file;
    final Map<Integer, GaeFileBufferEntity> buffers = new HashMap<Integer, GaeFileBufferEntity>();
    int bufferPos = 0;
    int currentBufferIndex = 0;

    GaeIndexMixin(final GaeFileEntity file) {
        this.file = file;
        loadBuffer();
    }

    /**
     * Returns total size of file
     *
     * @return Size of file
     */
    long length() {
        return file.size;
    }

    /**
     * Returns position in file
     *
     * @return Position in file
     */
    long getFilePointer() {
        return currentBufferIndex * BUFFER_SIZE + bufferPos;
    }

    /**
     * Advances to next buffer
     */
    void nextBuffer() {
        currentBufferIndex++;
        bufferPos = 0;
        loadBuffer();
    }

    /**
     * Loads current buffer
     * <p/>
     * Creates a new buffer if buffer with specified index doesn't exist
     */
    void loadBuffer() {
        if (buffers.get(currentBufferIndex) != null) {
            return;
        }

        GaeFileBufferEntity buffer = ofy().load().type(GaeFileBufferEntity.class).parent(file).id(currentBufferIndex + 1).now();

        if (buffer == null) {
            buffer = new GaeFileBufferEntity(file, currentBufferIndex + 1);
        }

        buffers.put(currentBufferIndex, buffer);
    }

    void flushBuffers() {
        ofy().save().entities(buffers.values()).now();
    }

    /**
     * Returns current buffer
     *
     * @return Current buffer
     */
    byte[] currentBuffer() {
        return buffers.get(currentBufferIndex).buffer;
    }
}
