/* Copyright 2013 Sven Jacobs
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.svenjacobs.lugaene;

/*
 * #%L
 * LuGAEne
 * %%
 * Copyright (C) 2013 Sven Jacobs
 * %%
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * #L%
 */


import org.apache.lucene.store.IndexOutput;

import java.io.IOException;

import static com.googlecode.objectify.ObjectifyService.ofy;
import static com.svenjacobs.lugaene.GaeFileBufferEntity.BUFFER_SIZE;

/**
 * Output from a {@link GaeFileEntity}
 *
 * @author Sven Jacobs
 */
public class GaeIndexOutput extends IndexOutput {

    private final GaeIndexMixin mixin;
    private boolean changedSinceFlush = false;

    GaeIndexOutput(final GaeFileEntity file) {
        this.mixin = new GaeIndexMixin(file);
    }

    @Override
    public void flush() throws IOException {
        if (changedSinceFlush) {
            setSize();
            ofy().save().entity(mixin.file).now();
            mixin.flushBuffers();
            changedSinceFlush = false;
        }
    }

    @Override
    public void close() throws IOException {
        flush();
    }

    @Override
    public long getFilePointer() {
        return mixin.getFilePointer();
    }

    @Override
    public void seek(final long pos) throws IOException {
        setSize();

        mixin.currentBufferIndex = (int) (pos / BUFFER_SIZE);
        mixin.bufferPos = (int) (pos % BUFFER_SIZE);

        mixin.loadBuffer();
    }

    @Override
    public long length() throws IOException {
        return mixin.length();
    }

    @Override
    public void writeByte(final byte b) throws IOException {
        changedSinceFlush = true;

        // current buffer is full, create new buffer
        if (mixin.bufferPos == BUFFER_SIZE) {
            mixin.nextBuffer();
        }

        mixin.currentBuffer()[mixin.bufferPos++] = b;
    }

    @Override
    public void writeBytes(final byte[] b,
                           int offset,
                           int length) throws IOException {

        changedSinceFlush = true;

        while (length > 0) {
            int remainingBytesInBuffer = BUFFER_SIZE - mixin.bufferPos;

            if (remainingBytesInBuffer == 0) {
                mixin.nextBuffer();
                remainingBytesInBuffer = BUFFER_SIZE;
            }

            final int bytesToCopy = Math.min(length, remainingBytesInBuffer);

            System.arraycopy(b, offset, mixin.currentBuffer(), mixin.bufferPos, bytesToCopy);

            offset += bytesToCopy;
            length -= bytesToCopy;
            mixin.bufferPos += bytesToCopy;
        }
    }

    private void setSize() {
        if (mixin.getFilePointer() > mixin.file.size) {
            mixin.file.size = mixin.getFilePointer();
        }
    }
}
