/* Copyright 2013 Sven Jacobs
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.svenjacobs.lugaene;

/*
 * #%L
 * LuGAEne
 * %%
 * Copyright (C) 2013 Sven Jacobs
 * %%
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * #L%
 */


import org.apache.lucene.store.Lock;
import org.apache.lucene.store.LockFactory;

import java.io.IOException;

import static com.googlecode.objectify.ObjectifyService.ofy;

/**
 * Factory that uses {@link GaeLockEntity} locks
 *
 * @author Sven Jacobs
 */
class GaeLockFactory extends LockFactory {

    private final GaeDirectoryEntity directory;

    GaeLockFactory(final GaeDirectoryEntity directory) {
        this.directory = directory;
    }

    @Override
    public Lock makeLock(final String lockName) {

        return new Lock() {
            @Override
            public boolean obtain() throws IOException {
                GaeLockEntity lock = loadLock(lockName);

                if (lock != null) {
                    if (lock.locked) {
                        return false;
                    }

                    lock.locked = true;
                    ofy().save().entity(lock).now();

                    return true;
                } else {
                    lock = new GaeLockEntity(directory, lockName);
                    lock.locked = true;
                    ofy().save().entity(lock).now();
                    return true;
                }
            }

            @Override
            public void release() throws IOException {
                final GaeLockEntity lock = loadLock(lockName);

                if (lock != null) {
                    lock.locked = false;
                    ofy().save().entity(lock).now();
                }
            }

            @Override
            public boolean isLocked() throws IOException {
                final GaeLockEntity lock = loadLock(lockName);
                return lock != null && lock.locked;
            }
        };
    }

    @Override
    public void clearLock(final String lockName) throws IOException {
        ofy().delete().type(GaeLockEntity.class).parent(directory).id(lockName).now();
    }

    private GaeLockEntity loadLock(final String lockName) {
        return ofy().load().type(GaeLockEntity.class).parent(directory).id(lockName).now();
    }
}
